@@ -9,19 +9,17 @@ use nom::{
 
 ///
 #[derive(Clone, Debug, PartialEq)]
-pub enum AstNode<'a> {
-    Root(Vec<AstNode<'a>>),
-    Styled(AstStyle<'a>),
-    Arbitrary(AstArbitrary<'a>),
-    SelfReference(AstReference),
-    Grouped(AstGroup<'a>),
+pub struct AstGroup<'a> {
+    head: AstStyle<'a>,
+    children: Vec<AstGroupItem<'a>>,
 }
 
 ///
 #[derive(Clone, Debug, PartialEq)]
-pub struct AstGroup<'a> {
-    head: AstStyle<'a>,
-    children: Vec<AstNode<'a>>,
+pub enum AstGroupItem<'a> {
+    Grouped(AstGroup<'a>),
+    Styled(AstStyle<'a>),
+    // SelfReference(AstReference),
 }
 
 ///
@@ -39,7 +37,7 @@ pub struct AstArbitrary<'a> {
     pub arbitrary: &'a str,
 }
 
-#[derive(Clone, Debug, PartialEq)]
+#[derive(Clone, Debug, PartialEq, Default)]
 pub struct AstElements<'a> {
     pub elements: Vec<&'a str>,
 }
