@@ -7,31 +7,16 @@ use nom::{
     sequence::delimited,
 };
 
-impl<'a> AstNode<'a> {
-    pub fn parse(input: &'a str) -> IResult<&'a str, Self> {
-        alt((Self::parse_self, Self::parse_style))(input)
-    }
-    #[inline]
-    fn parse_style(input: &'a str) -> IResult<&'a str, Self> {
-        AstStyle::parse(input).map(|(rest, ok)| (rest, Self::Styled(ok)))
-    }
-    #[inline]
-    fn parse_self(input: &'a str) -> IResult<&'a str, Self> {
-        AstReference::parse(input).map(|(rest, ok)| (rest, Self::SelfReference(ok)))
-    }
-}
-
 impl<'a> AstGroup<'a> {
     /// `v:a?(a(a b))`
     pub fn parse(input: &'a str) -> IResult<&'a str, Self> {
-        let (rest, (head, _, children)) =
-            tuple((AstStyle::parse, multispace0, Self::parse_many))(input)?;
+        let (rest, (head, children)) = tuple((AstStyle::parse, Self::parse_many))(input)?;
         Ok((rest, Self { head, children }))
     }
     #[inline]
-    fn parse_many(input: &'a str) -> IResult<&'a str, Vec<AstNode>> {
-        let head = AstNode::parse;
-        let rest = many0(tuple((multispace1, AstNode::parse)));
+    fn parse_many(input: &'a str) -> IResult<&'a str, Vec<AstGroupItem>> {
+        let head = AstGroupItem::parse;
+        let rest = many0(tuple((multispace1, AstGroupItem::parse)));
         let many = tuple((multispace0, head, rest, multispace0));
         let (rest, (_, first, other, _)) = delimited(char('('), many, char(')'))(input)?;
         let mut out = vec![first];
@@ -40,13 +25,31 @@ impl<'a> AstGroup<'a> {
     }
 }
 
+impl<'a> AstGroupItem<'a> {
+    pub fn parse(input: &'a str) -> IResult<&'a str, Self> {
+        alt((Self::parse_nested, Self::parse_style))(input)
+    }
+    #[inline]
+    fn parse_style(input: &'a str) -> IResult<&'a str, Self> {
+        AstStyle::parse(input).map(|(rest, ok)| (rest, Self::Styled(ok)))
+    }
+    // #[inline]
+    // fn parse_self(input: &'a str) -> IResult<&'a str, Self> {
+    //     AstReference::parse(input).map(|(rest, ok)| (rest, Self::SelfReference(ok)))
+    // }
+    #[inline]
+    fn parse_nested(input: &'a str) -> IResult<&'a str, Self> {
+        AstGroup::parse(input).map(|(rest, ok)| (rest, Self::Grouped(ok)))
+    }
+}
+
 impl<'a> AstStyle<'a> {
     /// `v:v::-?a-a-a-[A]`
     pub fn parse(input: &'a str) -> IResult<&'a str, Self> {
         let (rest, (variants, negative, elements, arbitrary)) = tuple((
             many0(ASTVariant::parse),
             opt(char('-')),
-            AstElements::parse,
+            opt(AstElements::parse),
             opt(AstArbitrary::parse),
         ))(input)?;
 
@@ -55,7 +58,7 @@ impl<'a> AstStyle<'a> {
             Self {
                 negative: negative.is_some(),
                 variants,
-                elements: elements.elements,
+                elements: elements.unwrap_or_default().elements,
                 arbitrary: arbitrary.map(|s| s.arbitrary),
             },
         ))
@@ -72,7 +75,7 @@ impl<'a> AstElements<'a> {
     }
     #[inline]
     fn parse_head(input: &'a str) -> IResult<&'a str, &'a str> {
-        let stop = |c: char| -> bool { matches!(c, ' ' | '-' | '[' | '(') };
+        let stop = |c: char| -> bool { matches!(c, ' ' | '-' | '[' | ']' | '(' | ')') };
         take_till1(stop)(input)
     }
     #[inline]
